USE Lib
GO

---------------------------------------------------------------------------
-- BEGIN MAINTENANCE WRAPPER
----------------------------
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'fn_recs_gen') BEGIN
	DROP  FUNCTION  tsql.fn_recs_gen
	PRINT '<<< DROP FUNCTION tsql.recs_gen - Completed with SUCCESS >>>'
END

GO
----------------------------
-- END MAINTENANCE WRAPPER
---------------------------------------------------------------------------

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO


/*================================================================================================
Name:			tsql.sp_recs_gen
Type:			User defined function 
Owner:					
Description:	Generates requested number of records 
Parameters:		@recs_to_gen = number of records to generate
				@rand_flag = generates a random number column to join with other tables and 
							 generate addresses, names, phone numbers, etc.
Dependens on:  	none
Usage:			SELECT * FROM tsql.fn_recs_gen(1000)

Other:			Uses LGC algorithm (Linear Congruential Generator) since RAND cannot be used in UDF
				http://en.wikipedia.org/wiki/Linear_congruential_generator 	
--------------------------------------------------------------------------------------------------
Project:		http://sqllib.codeplex.com/
Help:			http://sqllib.codeplex.com/wikipage?title=sp_recs_gen&referringTitle=Documentation		
Version:		20120822
SrcCtrlApp:		SVN
SrcCtrlPath:	http://sqllib.codeplex.com/SourceControl/list/changesets
--------------------------------------------------------------------------------------------------
History:		11/07/2008 - Adrian E Dudau - Function created
History:		12/16/2009 - Adrian E Dudau - Changed param @recs_to_gen to BIGINT 			
History:		12/16/2009 - Adrian E Dudau - Added random id feature, can be used to link to other tables 
											  and generate names, adresses, etc.  		
History:		08/22/2012 - Adrian E Dudau - Added references to project, SVN and Help											  	
================================================================================================*/

CREATE FUNCTION [tsql].[fn_recs_gen] (@recs_to_gen BIGINT, @rand_flag BIT)
RETURNS @Result TABLE (id BIGINT IDENTITY(1,1), rand_id BIGINT)
AS

BEGIN
	  
	DECLARE @r BIGINT = 1   
	/*Formula  Xn+1 == ((a * Xn + c) MOD m) */  
	DECLARE @a  BIGINT = 0 /*multiplier: a > 0 AND a < m*/
	DECLARE @m  BIGINT = 0 /*modulus: m > 0, m > Xo >= 0*/
	DECLARE @c  BIGINT = 0 /*increment: c >= 0 AND < m */
	DECLARE @Xn BIGINT = 0  /*max value*/

	SET @Xn = @recs_to_gen 

	WHILE @r <= @recs_to_gen BEGIN
		IF @rand_flag = 1 BEGIN 
			SET @m = DATEPART(SECOND, GETDATE()) * 1000 + DATEPART(MS,GETDATE())
			SET @a = DATEPART(SECOND , GETDATE()) % DATEPART(MINUTE , GETDATE())  
			SET @c = @c + 1
				
			INSERT INTO @Result (rand_id) 
			VALUES((@a * @Xn + @c) % @m);			
		END	
		ELSE BEGIN 
			INSERT INTO @Result (rand_id) VALUES (NULL);
		END
			
		SET @r = @r + 1	
	END /*WHILE*/

	-- NOTE: for values over 100,000 it may be worth 
	--		 to insert more that one row for performance improv
	
	RETURN 
END

GO

---------------------------------------------------------------------------
-- BEGIN MAINTENANCE WRAPPER
-- Verify completion
----------------------------
IF EXISTS(SELECT * FROM sysobjects WHERE name = 'fn_recs_gen') BEGIN 
	PRINT '<<< CREATE FUNCTION tsql.fn_recs_gen - Completed with SUCCESS >>>'
END

ELSE BEGIN 
	PRINT '<<< CREATE FUNCTION tsql.fn_recs_gen - Completed with ERROR >>>'
END


GO
----------------------------
-- END MAINTENANCE WRAPPER
---------------------------------------------------------------------------

--  GRANT SELECT TO PUBLIC will be used at Db level
